# -*- coding: utf-8 -*-
"""
URLs for the subscriber module.
"""
from django.conf.urls import patterns, url
from django.views.generic import TemplateView

urlpatterns = patterns(
    '',
    url(r'^thank-you/$', TemplateView.as_view(template_name="subscriber/thank_you.html"), name="thank_you"),
    url(
        r'^successful-unsubscribe/$',
        TemplateView.as_view(template_name="subscriber/successful_unsubscribe.html"),
        name="successful_unsubscribe"
    ),
)

urlpatterns += patterns(
    'holding_page.subscribers.admin_views',
    url(r'^admin/data/export/$', 'export_csv', name="export_csv"),
)

urlpatterns += patterns(
    'holding_page.subscribers.views',
    url(r'^unsubscribe/(?P<email>.+)/$', 'unsubscribe_form', name="unsubscribe_form_with_email"),
    url(r'^unsubscribe/$', 'unsubscribe_form', name="unsubscribe_form"),
    url(
        r'^(?P<code>[0-9A-Za-z]{8}-[0-9A-Za-z]{4}-[0-9A-Za-z]{4}-[0-9A-Za-z]{4}-[0-9A-Za-z]{12})/$',
        'subscriber_form',
        name="subscriber_form_with_code"
    ),
    url(r'^$', 'subscriber_form', name="subscriber_form"),
)



# -*- coding: utf-8 -*-
"""Views used by the characters admin."""
from __future__ import unicode_literals
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import permission_required
from django.http import HttpResponse
from django.template import loader, Context
from holding_page.subscribers.models import Subscriber


@staff_member_required
@permission_required('subscriber.export_csv')
def export_csv(request):
    """
    This admin view allows a staff member with the export_csv permission
    to produce a CSV of subscribed users
    """
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename=subscriber_data.csv'
    t = loader.get_template('data/subscriber_data.csv')
    c = Context({
        'subscribers': Subscriber.objects.all(),
    })
    response.write(t.render(c))
    return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Subscriber',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('full_name', models.CharField(max_length=255)),
                ('email', models.EmailField(unique=True, max_length=254, db_index=True)),
                ('share_code', models.CharField(max_length=10, null=True, db_index=True)),
                ('source_share_code', models.CharField(max_length=10, null=True, blank=True)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now=True)),
            ],
            options={
                'permissions': (('export_csv', 'Can export CSV data'),),
            },
        ),
    ]



# -*- coding: utf-8 -*-
"""
Tests for the subscribers app.
"""
from __future__ import unicode_literals
from django.contrib.auth.models import User, Permission
from django.core import mail
from django.core.urlresolvers import reverse
from django.test import TestCase
from mock import patch
from holding_page.subscribers.forms import SubscriberForm, UnsubscribeForm
from holding_page.subscribers.models import Subscriber


class SubscriptionFormTestCase(TestCase):
    """
    Collection of tests to ensure the subscription form works correctly
    """
    fixtures = ['test_data.json']

    def setUp(self):
        self.data = {
            'full_name': 'Test Name',
            'email': 'test@example.com'
        }

    def test_subscriber_model_has_unicode(self):
        """
        Tests that subscribers have a unicode display for the admin.
        """
        subscriber = Subscriber(email='test@example.com', full_name='Full Name')
        self.assertEquals(subscriber.__unicode__(), 'Full Name <test@example.com>')

    def test_form_renders(self):
        """
        Tests that the form renders correctly
        """
        response = self.client.get(reverse('subscriber:subscriber_form'))
        self.assertEqual(response.status_code, 200)
        self.assertTrue(isinstance(response.context['form'], SubscriberForm))

    def test_form_submits(self):
        """
        Tests that the form can be submitted with post, and the Fullname
        and email address are saved
        """
        response = self.client.post(reverse('subscriber:subscriber_form'),
                                    self.data)
        self.assertTrue(response.status_code, 302)
        self.assertRedirects(response, reverse('subscriber:thank_you'))
        Subscriber.objects.get(email='test@example.com', full_name='Test Name')

    def test_form_unique_email(self):
        """
        Tests that the form requires a unique email address
        """
        self.client.post(reverse('subscriber:subscriber_form'), self.data)
        response = self.client.post(reverse('subscriber:subscriber_form'),
                                    self.data)
        self.assertFormError(response,
                             "form",
                             'email',
                             'Subscriber with this Email address already exists.')

    def test_form_valid_email(self):
        """
        Tests that the form does not save details if the details provided
        are incorrect, i.e. badly formatted email address
        """
        data = {'email': 'test@example'}
        response = self.client.post(reverse('subscriber:subscriber_form'), data)
        self.assertTrue(response.status_code, 200)
        self.assertFormError(response, "form", 'email', 'Enter a valid email address.')

    def test_form_required_fields(self):
        """
        Tests that the form raises errors if required fields are not provided
        """
        data = {}
        response = self.client.post(reverse('subscriber:subscriber_form'), data)
        self.assertTrue(response.status_code, 200)
        self.assertFormError(response,
                             "form",
                             'email',
                             'This field is required.')
        self.assertFormError(response,
                             "form",
                             'full_name',
                             'This field is required.')

    def test_send_email(self):
        """
        Tests that an email is sent to new users, and that the email includes
        their sharing URL
        """
        self.client.post(reverse('subscriber:subscriber_form'), self.data)
        self.assertEqual(len(mail.outbox), 1)

    @patch('holding_page.subscribers.models.uuid4')
    def test_subscription_share_code(self, uuid4):
        """
        Tests that every new user has a share code generated and saved
        """
        uuid4.return_value = '68b475a0-5e1c-4a3a-8eae-db469acd65a6'
        self.client.post(reverse('subscriber:subscriber_form'), self.data)
        subscriber = Subscriber.objects.get(email='test@example.com', full_name='Test Name')
        self.assertEquals(subscriber.share_code, '68b475a0-5e1c-4a3a-8eae-db469acd65a6')

    def test_form_renders_with_share_code(self):
        """
        Tests that a form submitted with a share code correctly saves the
        reference to this share code
        """
        referrer = Subscriber.objects.get(pk=1)
        response = self.client.get(reverse('subscriber:subscriber_form_with_code', args=[referrer.share_code]))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['form'].initial['source_share_code'], referrer.share_code)

    def test_form_valid_share_code(self):
        """
        Tests that a share code entered is actually valid, raises an error if not
        """
        self.data['source_share_code'] = '1234567890'
        response = self.client.post(reverse('subscriber:subscriber_form'),
                                    self.data)
        self.assertEqual(response.status_code, 200)
        self.assertFormError(response,
                             "form",
                             'source_share_code',
                             'Invalid share code.')

    def test_remove_email_address_renders(self):
        """
        Tests that the unsubscribe form renders
        """
        response = self.client.get(reverse('subscriber:unsubscribe_form'))
        self.assertEqual(response.status_code, 200)
        self.assertTrue(isinstance(response.context['form'], UnsubscribeForm))

    def test_remove_email_address(self):
        """
        Tests that posting an email to the unsubscribe remove that user's entry
        """
        data = {'email': 'testdata@example.com'}
        response = self.client.post(reverse('subscriber:unsubscribe_form'), data)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, reverse('subscriber:successful_unsubscribe'))

    def test_unsubscribe_form_valid_email_address(self):
        """
        Tests that a share code entered is actually valid, raises an error if not
        """
        self.data['email'] = 'blah@example.com'
        response = self.client.post(reverse('subscriber:unsubscribe_form'), self.data)
        self.assertFormError(response, 'form', 'email', 'Invalid email address.')

    def test_remove_email_address_renders_with_email(self):
        """
        Tests that the unsubscribe form renders with a provided email address
        """
        response = self.client.get(reverse('subscriber:unsubscribe_form_with_email',
                                           args=['testdata@example.com']))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['form'].initial['email'], 'testdata@example.com')

    def test_export_data(self):
        """
        Tests that the export data admin view correctly renders a CSV
        """
        staff_user = User.objects.create_superuser(
            username='staff_user',
            email="staff_user@example.com",
            password='staff_user'
        )
        export_csv = Permission.objects.get(codename='export_csv')
        staff_user.user_permissions.add(export_csv)
        staff_user.save()
        self.client.login(username="staff_user", password="staff_user")
        response = self.client.get(reverse('subscriber:export_csv'))
        self.assertEqual(response.content, b'"Test Data User","testdata@example.com","0"\n')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('subscribers', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscriber',
            name='share_code',
            field=models.CharField(db_index=True, max_length=36, null=True),
        ),
        migrations.AlterField(
            model_name='subscriber',
            name='source_share_code',
            field=models.CharField(db_index=True, max_length=36, blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Holds the subscriber model which represents someone who has subscribed to updates.
"""
from uuid import uuid4
from django.db import models
from django.db.models.signals import post_save
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.utils.translation import ugettext as _


class Subscriber(models.Model):
    """
    A subscriber represents a visitor to the holding page who has subscribed by providing their email address.
    """
    full_name = models.CharField(_('Full name'), max_length=255)
    email = models.EmailField(_('Email address'), unique=True, db_index=True)
    share_code = models.CharField(_('Share code'), max_length=36, null=True, db_index=True)
    source_share_code = models.CharField(_('Source share code'), max_length=36, blank=True, null=True, db_index=True)
    date_created = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return '{0} <{1}>'.format(self.full_name, self.email)

    class Meta(object):
        permissions = (
            ('export_csv', _('Can export CSV data')),)

    @property
    def subscribers_recruited(self):
        return Subscriber.objects.filter(source_share_code=self.share_code).count()

    @staticmethod
    def share_code_generator(sender, **kwargs):
        """
        Generates a unique share code for a subscriber using a ask for
        forgiveness, not permission, system
        """
        del sender
        instance = kwargs['instance']
        if kwargs['created'] is False:
            return
        instance.share_code = uuid4()
        instance.save()

    @staticmethod
    def send_email(sender, **kwargs):
        """
        Sends a welcome email to the user informing them about the share
        code, and also information about un-subscribing
        """
        del sender
        instance = kwargs['instance']
        if kwargs['created'] is False:
            return
        site = Site.objects.get(pk=settings.SITE_ID)
        subject = render_to_string(
            'email/welcome_subject.txt',
            {'subscriber':  instance, 'site': site}
        )
        body = render_to_string(
            'email/welcome_body.txt',
            {'subscriber':  instance, 'site': site}
        )
        send_mail(
            subject,
            body,
            settings.DEFAULT_FROM_EMAIL,
            [instance.email]
        )


post_save.connect(Subscriber.share_code_generator, sender=Subscriber)
post_save.connect(Subscriber.send_email, sender=Subscriber)



# -*- coding: utf-8 *-*
"""
Adds admin configuration for subscribers.
"""
from django.contrib import admin
from holding_page.subscribers.models import Subscriber


class SubscriberAdmin(admin.ModelAdmin):
    list_display = ('full_name', 'email', 'subscribers_recruited')
admin.site.register(Subscriber, SubscriberAdmin)



# -*- coding: utf-8 -*-
"""
Views for subscribers. For views in Django admin see admin_views.py
"""
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from holding_page.subscribers.forms import SubscriberForm, UnsubscribeForm
from holding_page.subscribers.models import Subscriber


def subscriber_form(request, code=None):
    """
    The default view for the app, simply provides a subscriber form
    that saves a name and email address to the subscriber database.

    If a share code is provided then it will check for a matching subscriber and record that the
    user provided that share code.

    :param request: The request object.
    :type request: HttpRequest

    :param code: A share code - this will be included if the user is accessing the
       site using a shared by an other subscriber.
    :type code: unicode
    """
    initial = {}
    if code is not None:
        get_object_or_404(Subscriber, share_code=code)
        initial['source_share_code'] = code

    if request.method == 'POST':
        form = SubscriberForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('subscriber:thank_you'))
    else:
        form = SubscriberForm(initial=initial)

    context = {'form': form}
    return render_to_response(
        'subscriber/subscriber_form.html',
        context,
        RequestContext(request)
    )


def unsubscribe_form(request, email=None):
    """
    View to handle unsubscriptions.

    :param request: The request object.
    :type request: HttpRequest

    :param email: The email address of the subscriber to unsubscribe.
    :type email: unicode
    """
    initial = {}
    if email is not None:
        get_object_or_404(Subscriber, email=email)
        initial['email'] = email

    if request.method == 'POST':
        form = UnsubscribeForm(request.POST)
        if form.is_valid():
            Subscriber.objects.get(email=form.cleaned_data['email']).delete()
            return HttpResponseRedirect(reverse('subscriber:successful_unsubscribe'))
    else:
        form = UnsubscribeForm(initial=initial)

    return render_to_response(
        'subscriber/unsubscribe_form.html',
        {'form': form},
        RequestContext(request),
    )



# -*- coding: utf-8 -*-
"""
Forms used to create subscribers and allow them to unsubscribe.
"""
from holding_page.subscribers.models import Subscriber
from django import forms
from django.utils.translation import ugettext as _


class SubscriberForm(forms.ModelForm):
    """
    Forms used to create a subscriber.
    """
    class Meta(object):
        model = Subscriber
        fields = ['full_name', 'email', 'source_share_code']
        widgets = {
            'source_share_code': forms.HiddenInput(),
        }

    def clean_source_share_code(self):
        """
        Looks for the share code and raises a validation error if it is not valid.
        """
        if self.cleaned_data['source_share_code'] != '':
            if not Subscriber.objects.filter(share_code=self.cleaned_data['source_share_code']).exists():
                del self.cleaned_data['source_share_code']
                raise forms.ValidationError(_('Invalid share code.'))
        return self.cleaned_data['source_share_code']


class UnsubscribeForm(forms.Form):
    """
    Form used to unsubscribe and subscriber.
    """
    email = forms.CharField(label=_('Email address'))

    def clean_email(self):
        if not Subscriber.objects.filter(email=self.cleaned_data['email']).exists():
            del self.cleaned_data['email']
            raise forms.ValidationError(_('Invalid email address.'))
        return self.cleaned_data['email']



from distutils.core import setup
from holding_page import __version__

version = __version__
setup(
    name='django-holding-page',
    version=version,
    packages=['holding_page', 'holding_page.subscribers', 'holding_page.subscribers.migrations'],
    url='https://github.com/danux/django-holding-page/',
    license='MIT',
    author='Daniel Davies',
    author_email='danieldavies127@gmail.com',
    description='A viral holding page to collect email address with export and unsubscribe functionality.'
)
